import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;
public class Engine extends Canvas{
    Group start;
    Group end;
    int w;
    int h;
    boolean keypressed=false;
    int keycode=0;
    Group direct;
    String status="";
    World world;
    Light light;
    Background bg;
    Camera camera;
    Graphics3D graphics3d;
    Transform transform;
    public Engine(){
        w=getWidth();
        h=getHeight();
        direct=new Group();
        world=new World();
        light=new Light();
        light.setMode(Light.AMBIENT);
        world.addChild(light);
        camera=new Camera();
        camera.setPerspective(50,((float)w/(float)h),1.5f,33000.0f);
        world.addChild(camera);
        world.setActiveCamera(camera);
        bg=new Background();
        graphics3d=Graphics3D.getInstance();
        transform=new Transform();
    }
    public float[] convert(float[][]pro){
        float[]ret=new float[16];
        ret[0]=pro[0][0];
        ret[1]=pro[0][1];
        ret[2]=pro[0][2];
        ret[3]=pro[0][3];
        ret[4]=pro[1][0];
        ret[5]=pro[1][1];
        ret[6]=pro[1][2];
        ret[7]=pro[1][3];
        ret[8]=pro[2][0];
        ret[9]=pro[2][1];
        ret[10]=pro[2][2];
        ret[11]=pro[2][3];
        ret[12]=pro[3][0];
        ret[13]=pro[3][1];
        ret[14]=pro[3][2];
        ret[15]=pro[3][3];
        return ret;
    }
    public float[] normalize(float[]v){
        float[]ret=new float[3];
        float len=(float)Math.sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
        ret[0]=v[0]/len;
        ret[1]=v[1]/len;
        ret[2]=v[2]/len;
        return ret;
    }
    public float[] vectorproduct(float[]a,float[]b){
        float[]ret=new float[3];
        ret[0]=a[1]*b[2]-a[2]*b[1];
        ret[1]=a[2]*b[0]-a[0]*b[2];
        ret[2]=a[0]*b[1]-a[1]*b[0];
        return ret;
    }
    public float[][] projection(float eyex,float eyey,float eyez,float centerx,float centery,float centerz,float upx,float upy,float upz){
        float[]f=new float[3];
        f[0]=centerx-eyex;
        f[1]=centery-eyey;
        f[2]=centerz-eyez;
        float[]f2=normalize(f);
        float[]up=new float[3];
        up[0]=upx;
        up[1]=upy;
        up[2]=upz;
        float[]up2=normalize(up);
        float[]s=vectorproduct(f2,up2);
        float[]u=vectorproduct(s,f2);
        float[][]ret=new float[4][4];
        ret[0][0]=s[0];
        ret[0][1]=s[1];
        ret[0][2]=s[2];
        ret[0][3]=0;
        ret[1][0]=u[0];
        ret[1][1]=u[1];
        ret[1][2]=u[2];
        ret[1][3]=0;
        ret[2][0]=-f2[0];
        ret[2][1]=-f2[1];
        ret[2][2]=-f2[2];
        ret[2][3]=0;
        ret[3][0]=0;
        ret[3][1]=0;
        ret[3][2]=0;
        ret[3][3]=1;
        return ret;
    }
    public void paint(Graphics g){
        if(world==null){
            return;
        }
        graphics3d.bindTarget(g);
        try{
            graphics3d.clear(bg);
            graphics3d.render(world);
        }
        catch(Exception exv){
            System.out.println("exv: "+exv);
        }
        graphics3d.releaseTarget();
        if(!"".equals(status)){
            g.setColor(0);
            g.fillRect(0,0,w,15);
            g.setColor(255*256*256+255*256+255);
            g.drawLine(0,15,w,15);
            g.drawString(status,10,0,0);
        }
    }
    public void setCam(Vektor position,float wx,float wy){
	float z=(float)Math.cos(wy);
	float w=(float)Math.sin(wy);
	float y=(float)Math.sin(wx)*w;
	float x=(float)Math.cos(wx)*w;
        double wy2=wy+Math.PI/2;
        double wx2=wx;
	float z2=(float)Math.cos(wy2);
	float w2=(float)Math.sin(wy2);
	float y2=(float)Math.sin(wx2)*w2;
	float x2=(float)Math.cos(wx2)*w2;
        float[][]pro=projection(position.x,position.y,position.z,position.x+x,position.y+y,position.z+z,x2,y2,z2);
        float[]pro2=convert(pro);
        Transform mult=new Transform();
        mult.set(pro2);
        camera.setTranslation(0,0,0);
        camera.setPerspective(50,1.0f,1.5f,30000.0f);
        int type=camera.getProjection(transform);
        transform.postMultiply(mult);
        transform.postTranslate(-position.x,-position.y,-position.z);
        camera.setGeneric(transform);
    }
    public void setStatus(String status){
        this.status=status;
    }
    public void add(Polygon pol){
        if(pol.mesh!=null){
            world.addChild(pol.mesh);
            direct.add(pol);
        }
    }
    public void add(Kugel kugel){
        if(kugel.mesh!=null){
            world.addChild(kugel.mesh);
            direct.add(kugel);
        }
    }
    public void remove(Polygon pol){
        direct.remove(pol);
        if(pol.mesh!=null){
            world.removeChild(pol.mesh);
        }
    }
    public void remove(Kugel kugel){
        direct.remove(kugel);
        if(kugel.mesh!=null){
            world.removeChild(kugel.mesh);
        }
    }
    public void add(Group g){
        if(start==null){
            start=g;
            end=g;
        }
        else{
            end.next=g;
            end=end.next;
        }
        Polygon cur=g.start;
        while(cur!=null){
            if(cur.mesh!=null){
                world.addChild(cur.mesh);
            }
            cur=cur.next;
        }
        Kugel curb=g.startb;
        while(curb!=null){
            if(curb.mesh!=null){
                world.addChild(curb.mesh);
            }
            curb=curb.next;
        }
    }
    public void remove(Group g){
        Group cur=start;
        Group fore=null;
        while(cur!=null){
            boolean b=true;
            if(cur==g){
                if(fore==null){
                    if(end==start){
                        end=null;
                        start=null;
                    }
                    else{
                        start=start.next;
                    }
                }
                else{
                    fore.next=cur.next;
                    b=false;
                }
            }
            if(b){
                fore=cur;
                cur=cur.next;
            }
            else{
                cur=cur.next;
            }
        }
        Polygon curb=g.start;
        while(curb!=null){
            if(curb.mesh!=null){
                world.removeChild(curb.mesh);
            }
            curb=curb.next;
        }
        Kugel curb2=g.startb;
        while(curb2!=null){
            if(curb2.mesh!=null){
                world.removeChild(curb2.mesh);
            }
            curb2=curb2.next;
        }
    }
    public void transform(float x,float y,float z){
        Group cur=start;
        while(cur!=null){
            cur.transform(x,y,z);
            cur=cur.next;
        }
        direct.transform(x,y,z);
    }
    public void rotLeft(float w){
        Group cur=start;
        while(cur!=null){
            cur.rotLeft(w);
            cur=cur.next;
        }
        direct.rotLeft(w);
    }
    public void rotUp(float w){
        Group cur=start;
        while(cur!=null){
            cur.rotUp(w);
            cur=cur.next;
        }
        direct.rotUp(w);
    }
    public void keyPressed(int keycode){
        keypressed=true;
        this.keycode=keycode;
    }
    public boolean keypressed(){
        return keypressed;
    }
    public int getKeyCode(){
        if(!keypressed){
            return 0;
        }
        keypressed=false;
        return keycode;
    }
}
class Group{
    Polygon start;
    Polygon end;
    Kugel startb;
    Kugel endb;
    Group next;
    public Group(){
    }
    public void add(Polygon pol){
        if(start==null){
            start=pol;
            end=pol;
        }
        else{
            end.next=pol;
            end=end.next;
        }
    }
    public void remove(Polygon pol){
        Polygon cur=start;
        Polygon fore=null;
        while(cur!=null){
            boolean b=true;
            if(cur==pol){
                if(fore==null){
                    if(end==start){
                        end=null;
                        start=null;
                    }
                    else{
                        start=start.next;
                    }
                }
                else{
                    fore.next=cur.next;
                    b=false;
                }
            }
            if(b){
                fore=cur;
                cur=cur.next;
            }
            else{
                cur=cur.next;
            }
        }
    }
    public void add(Kugel kugel){
        if(startb==null){
            startb=kugel;
            endb=kugel;
        }
        else{
            endb.next=kugel;
            endb=endb.next;
        }
    }
    public void remove(Kugel kugel){
        Kugel curb=startb;
        Kugel fore=null;
        while(curb!=null){
            boolean b=true;
            if(curb==kugel){
                if(fore==null){
                    if(endb==startb){
                        endb=null;
                        startb=null;
                    }
                    else{
                        startb=startb.next;
                    }
                }
                else{
                    fore.next=curb.next;
                    b=false;
                }
            }
            if(b){
                fore=curb;
                curb=curb.next;
            }
            else{
                curb=curb.next;
            }
        }
    }
    public void transform(float x,float y,float z){
        Polygon cur=start;
        while(cur!=null){
            cur.transform(x,y,z);
            cur=cur.next;
        }
        Kugel curb=startb;
        while(curb!=null){
            curb.transform(x,y,z);
            curb=curb.next;
        }
    }
    public void rotLeft(float w){
        Polygon cur=start;
        while(cur!=null){
            cur.rotLeft(w);
            cur=cur.next;
        }
        Kugel curb=startb;
        while(curb!=null){
            curb.rotLeft(w);
            curb=curb.next;
        }
    }
    public void rotUp(float w){
        Polygon cur=start;
        while(cur!=null){
            cur.rotUp(w);
            cur=cur.next;
        }
        Kugel curb=startb;
        while(curb!=null){
            curb.rotUp(w);
            curb=curb.next;
        }
    }
}
class Polygon{
    Polygon next;
    Vektor v1;
    Vektor v2;
    Vektor v3;
    Vektor v4;
    Mesh mesh;
    Image img;
    Image2D img2;
    Transform transform;
    boolean triangle=false;
    short[]pos;
    int[]slens;
    short[]texval;
    TriangleStripArray tsa;
    VertexBuffer vb;
    VertexArray va;
    VertexArray vc;
    Appearance app;
    Texture2D tex;
    PolygonMode pm;
    int w;
    int h;
    int wb;
    int hb;
    public Polygon(Vektor v1,Vektor v2,Vektor v3,Vektor v4,Image img){
        this.v1=v1;
        this.v2=v2;
        this.v3=v3;
        this.v4=v4;
        this.img=img;
        w=img.getWidth();
        h=img.getHeight();
        while(w==-1||h==-1){
            w=img.getWidth();
            h=img.getHeight();
            try{
                Thread.yield();
            }
            catch(Exception e){
            }
        }
        wb=1;
        while(wb<w){
            wb=wb*2;
        }
        hb=1;
        while(hb<h){
            hb=hb*2;
        }
        int[]rgb=new int[w*h];
        img.getRGB(rgb,0,w,0,0,w,h);
        int[]extend=new int[wb*hb];
        for(int i=0;i<w;i++){
            for(int j=0;j<h;j++){
                extend[j*wb+i]=rgb[j*w+i];
            }
        }
        Image imgb=Image.createRGBImage(extend,wb,hb,true);
        img2=new Image2D(Image2D.RGBA,imgb);
        buildq();
    }
    public Polygon(Vektor v1,Vektor v2,Vektor v3,Image img){
        triangle=true;
        this.v1=v1;
        this.v2=v2;
        this.v3=v3;
        this.img=img;
        w=img.getWidth();
        h=img.getHeight();
        while(w==-1||h==-1){
            w=img.getWidth();
            h=img.getHeight();
            try{
                Thread.yield();
            }
            catch(Exception e){
            }
        }
        wb=1;
        while(wb<w){
            wb=wb*2;
        }
        hb=1;
        while(hb<h){
            hb=hb*2;
        }
        int[]rgb=new int[w*h];
        img.getRGB(rgb,0,w,0,0,w,h);
        int[]extend=new int[wb*hb];
        for(int i=0;i<w;i++){
            for(int j=0;j<h;j++){
                extend[j*wb+i]=rgb[j*w+i];
            }
        }
        Image imgb=Image.createRGBImage(extend,wb,hb,true);
        img2=new Image2D(Image2D.RGBA,imgb);
        buildt();
    }
    public void buildt(){
        try{
            pos=new short[9];
            pos[0]=v1.x;
            pos[1]=v1.y;
            pos[2]=v1.z;
            pos[3]=v2.x;
            pos[4]=v2.y;
            pos[5]=v2.z;
            pos[6]=v3.x;
            pos[7]=v3.y;
            pos[8]=v3.z;
            slens=new int[1];
            slens[0]=3;
            texval=new short[6];
            texval[0]=0;
            texval[1]=(short)h;
            texval[2]=(short)w;
            texval[3]=(short)h;
            texval[4]=0;
            texval[5]=0;
            tsa=new TriangleStripArray(0,slens);
            vb=new VertexBuffer();
            va=new VertexArray(3,3,2);
            va.set(0,3,pos);
            vb.setPositions(va,1.0f,null);
            vc=new VertexArray(3,2,2);
            vc.set(0,3,texval);
            float[]bias=new float[2];
            bias[1]=((float)h)/((float)hb)-((float)h)/((float)wb);
            vb.setTexCoords(0,vc,1.0f/wb,bias);
            app=new Appearance();
            tex=new Texture2D(img2);
            app.setTexture(0,tex);
            pm=new PolygonMode();
            pm.setCulling(PolygonMode.CULL_NONE);
            app.setPolygonMode(pm);
            mesh=new Mesh(vb,tsa,app);
            mesh.setAppearance(0,app);
            transform=new Transform();
        }
        catch(Exception e){
        }
    }
    public void buildq(){
        try{
            pos=new short[18];
            pos[0]=v1.x;
            pos[1]=v1.y;
            pos[2]=v1.z;
            pos[3]=v2.x;
            pos[4]=v2.y;
            pos[5]=v2.z;
            pos[6]=v4.x;
            pos[7]=v4.y;
            pos[8]=v4.z;
            pos[9]=v2.x;
            pos[10]=v2.y;
            pos[11]=v2.z;
            pos[12]=v3.x;
            pos[13]=v3.y;
            pos[14]=v3.z;
            pos[15]=v4.x;
            pos[16]=v4.y;
            pos[17]=v4.z;
            slens=new int[2];
            slens[0]=3;
            slens[1]=3;
            texval=new short[12];
            texval[0]=0;
            texval[1]=(short)h;
            texval[2]=(short)w;
            texval[3]=(short)h;
            texval[4]=0;
            texval[5]=0;
            texval[6]=(short)w;
            texval[7]=(short)h;
            texval[8]=(short)w;
            texval[9]=0;
            texval[10]=0;
            texval[11]=0;
            tsa=new TriangleStripArray(0,slens);
            vb=new VertexBuffer();
            va=new VertexArray(6,3,2);
            va.set(0,6,pos);
            vb.setPositions(va,1.0f,null);
            vc=new VertexArray(6,2,2);
            vc.set(0,6,texval);
            float[]bias=new float[2];
            bias[1]=((float)h)/((float)hb)-((float)h)/((float)wb);
            vb.setTexCoords(0,vc,1.0f/wb,bias);
            app=new Appearance();
            tex=new Texture2D(img2);
            app.setTexture(0,tex);
            pm=new PolygonMode();
            pm.setCulling(PolygonMode.CULL_NONE);
            app.setPolygonMode(pm);
            mesh=new Mesh(vb,tsa,app);
            mesh.setAppearance(0,app);
            transform=new Transform();
        }
        catch(Exception e){
        }
    }
    public void transform(float x,float y,float z){
        Transform tf=new Transform();
        tf.postTranslate(x,y,z);
        MatMult.multleft(transform,tf);
        mesh.setTransform(transform);
    }
    public void rotLeft(float w){
        float w2=(float)(w*180/Math.PI);
        Transform tf=new Transform();
        tf.postRotate(w2,0.0f,1.0f,0.0f);
        MatMult.multleft(transform,tf);
        mesh.setTransform(transform);
    }
    public void rotUp(float w){
        float w2=(float)(w*180/Math.PI);
        Transform tf=new Transform();
        tf.postRotate(w2,1.0f,0.0f,0.0f);
        MatMult.multleft(transform,tf);
        mesh.setTransform(transform);
    }
}
class Kugel{
    Kugel next;
    short[]pos=new short[3*3*2*256];
    TriangleStripArray tsa;
    VertexBuffer vb;
    VertexArray colors;
    Appearance app;
    int[]slens=new int[2*256];
    Transform transform;
    Mesh mesh;
    short radius;
    double[]posf=new double[3*3*2*256];
    public Kugel(Vektor middle,int radiusb,int r,int g,int b){
        radius=(short)radiusb;
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                double wx=((double)i/16.0)*Math.PI*2;
                double wy=((double)j/16.0)*Math.PI-Math.PI/2;
                double wxp=((double)(i+1)/16.0)*Math.PI*2;
                double wyp=((double)(j+1)/16.0)*Math.PI-Math.PI/2;
                double zu=Math.sin(wyp)*radius;
                double zd=Math.sin(wy)*radius;
                double hu=Math.cos(wyp)*radius;
                if(hu<0){
                    hu=-hu;
                }
                double hd=Math.cos(wy)*radius;
                if(hd<0){
                    hd=-hd;
                }
                double xuu=Math.cos(wxp)*hu;
                double yuu=Math.sin(wxp)*hu;
                double xdd=Math.cos(wx)*hd;
                double ydd=Math.sin(wx)*hd;
                double xud=Math.cos(wx)*hu;
                double yud=Math.sin(wx)*hu;
                double zud=zu;
                double xdu=Math.cos(wxp)*hd;
                double ydu=Math.sin(wxp)*hd;
                double zdu=zd;
                posf[(i*16+j)*18]=xdd;
                posf[(i*16+j)*18+1]=ydd;
                posf[(i*16+j)*18+2]=zd;
                posf[(i*16+j)*18+3]=xuu;
                posf[(i*16+j)*18+4]=yuu;
                posf[(i*16+j)*18+5]=zu;
                posf[(i*16+j)*18+6]=xud;
                posf[(i*16+j)*18+7]=yud;
                posf[(i*16+j)*18+8]=zud;

                posf[(i*16+j)*18+9]=xdd;
                posf[(i*16+j)*18+10]=ydd;
                posf[(i*16+j)*18+11]=zd;
                posf[(i*16+j)*18+12]=xuu;
                posf[(i*16+j)*18+13]=yuu;
                posf[(i*16+j)*18+14]=zu;
                posf[(i*16+j)*18+15]=xdu;
                posf[(i*16+j)*18+16]=ydu;
                posf[(i*16+j)*18+17]=zdu;
            }
        }
        for(int i=0;i<3*3*2*16*16;i++){
            pos[i]=(short)posf[i];
        }
        for(int i=0;i<2*16*16;i++){
            slens[i]=3;
        }
        tsa=new TriangleStripArray(0,slens);
        vb=new VertexBuffer();
        colors=new VertexArray(3*2*16*16,3,1);
        byte[]colbyte=new byte[3*3*2*16*16];
        for(int i=0;i<3*2*16*16;i++){
            colbyte[i*3]=(byte)r;
            colbyte[i*3+1]=(byte)g;
            colbyte[i*3+2]=(byte)b;
        }
        colors.set(0,3*2*16*16,colbyte);
        vb.setColors(colors);
        VertexArray va=new VertexArray(3*2*16*16,3,2);
        va.set(0,3*2*16*16,pos);
        vb.setPositions(va,1.0f,new float[3]);
        app=new Appearance();
        PolygonMode pm=new PolygonMode();
        pm.setCulling(PolygonMode.CULL_NONE);
        app.setPolygonMode(pm);
        mesh=new Mesh(vb,tsa,app);
        transform=new Transform();
        transform.postTranslate(middle.x,middle.y,middle.z);
    }
    public void transform(float x,float y,float z){
        Transform tf=new Transform();
        tf.postTranslate(x,y,z);
        MatMult.multleft(transform,tf);
        mesh.setTransform(transform);
    }
    public void rotLeft(float w){
        float w2=(float)(w*180/Math.PI);
        Transform tf=new Transform();
        tf.postRotate(w2,0.0f,1.0f,0.0f);
        MatMult.multleft(transform,tf);
        mesh.setTransform(transform);
    }
    public void rotUp(float w){
        float w2=(float)(w*180/Math.PI);
        Transform tf=new Transform();
        tf.postRotate(w2,1.0f,0.0f,0.0f);
        MatMult.multleft(transform,tf);
        mesh.setTransform(transform);
    }
}
class Vektor{
    short x;
    short y;
    short z;
    public Vektor(int x,int y,int z){
        this.x=(short)x;
        this.y=(short)y;
        this.z=(short)z;
    }
}
class MatMult{
    public static void multleft(Transform original,Transform mult){
        float[]a=new float[16];
        mult.get(a);
        float[][]a2=new float[4][4];
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                a2[i][j]=a[i*4+j];
            }
        }
        float[]b=new float[16];
        original.get(b);
        float[][]b2=new float[4][4];
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                b2[i][j]=b[i*4+j];
            }
        }
        float[][]c2=new float[4][4];
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                for(int k=0;k<4;k++){
                    c2[i][j]=c2[i][j]+a2[i][k]*b2[k][j];
                }
            }
        }
        float[]c=new float[16];
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                c[i*4+j]=c2[i][j];
            }
        }
        original.set(c);
    }
}
